
#include <stdlib.h>
#include <math.h>

// May need to change this include to customize to your environment
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "st.h"



// Holds name of font and text filenames
char fontFilename[80];
char wordsFilename[80];


// Current size of the OpenGL window
int windowSizeX;
int windowSizeY;


// The following are parameters that are used to compute the shapes
// of the text regions for the two columns.

enum ColumnStyle {STRAIGHT, SINUSOID, TRIANGLE, DIAGONAL, INVALID};

ColumnStyle columnStyle;
float marginSize = 10.0f;
float columnSplit = 0.0f;

// determines whether the column borders are displayed
bool displayGuidelines = true;

// The scene graph
STScene scene;


STFontFace* font1 = NULL;
STFontFace* font2 = NULL;


// Destroys drawable objects in the scene graph
void destroyDrawables(STSceneNode* node) {

	if (node->GetObj())
		delete node->GetObj();

	STSceneNode* n = node->GetFirstChild();
	while (n) {
		destroyDrawables(n);
		n = n->GetNext();
	}
}


void parseText() {

	//
	// CS148 TODO
	// This function should read the file 'wordsFilename' and parse all words in all
	// paragraphs.  A line containing no words signifies a new paragraph.  For example
	// a file contain the following text would be parsed to consist of 4 paragraphs:
    //

	// This is an Example
	//
	// This is an example for CS148.  Students will need
	// to parse text files to produce a list of words.  This is the last
	// sentance of the second paragraph.
	//
	// This is the beginning of the third paragraph.  The third
	// paragraph consists of two sentances.  
	//
	// This is the fourth paragraph in this example.  Recall that the
	// first paragraph should be treated as the title, and therefore
	// should be centered and rendered with a font twice twice that of
	// the rest of the paragraphs.  All other paragraphs should be full
	// justified.  This is the end of the example.

}



// this function shuold be called every time the text needs to be reflowed.  Invocations
// of reflowText() are typically triggered by GLUT events, like a window resizse.
void reflowText() {


	// First destroy the existing scene graph and all assoicated objects
	// We will build the scene graph again from scratch.
	destroyDrawables(scene.GetRoot());
	scene.Clear();


	//
	// CS148 TODO
	// This function should now reflow the text given the current
	// state of the columns and font characteristics
	//


	// The following examples demonstrate how to add some STText to the scene.

	// Initialize two font faces.  Sizes are in points.
	font1 = new STFontFace("arial.ttf", 48);
	font2 = new STFontFace("times.ttf", 18);


	// Example 1: Here is an example of using the STText class as a simple
	// text box.  It renders the string "Large yellow text!" in yellow.
	STText* text1 = new STText(font1, "Large yellow text!");
	text1->SetColor( STColor(0.9f, 0.9f, 0.2f) );
	STSceneNode* root = scene.GetRoot();
	STSceneNode* node = root->PlaceObj( text1 );
	node->SetTransform( STTransform::Translate(marginSize, 175.0f, 0.0f) );


	// Example 2: Now, we'll do something a little closer to what is expected in
	// assignment 3.  We create one STText object per-word in the array
	// 'sentance', and then position them together in the scene graph
	// to render a complete sentance.

	char* sentance[] = { "This", "is", "a", "sentence", "made", "from", "many", "STText", "objects." };
	int numWords = sizeof(sentance) / sizeof(char*);

	float offsetX = marginSize;
	for (int i=0; i<numWords; i++) {

		STText* text = new STText(font2, sentance[i]);
		text->SetColor( STColor(1.0f, 1.0f, 1.0f) );
		node = root->PlaceObj( text );
		node->SetTransform( STTransform::Translate(offsetX, 100.0f, 0.0f) );

		// Add 5 pixels between words for spaces.  The width of the contexts of
		// a text box is returned by GetWidth() field of STText. To adjust the
		// spacing between words to ensure the text is displayed with full justification.
		offsetX += text->GetWidth() + 5.0f;
	}


	// Example 3: Now we center the string "CS148" in the middle of the screen:
	STText* text = new STText(font1, "CS148");
	text->SetColor( STColor(1.0f, 1.0f, 1.0f) );
	node = root->PlaceObj( text );
	node->SetTransform( STTransform::Translate( (windowSizeX-text->GetWidth()) / 2.0f, 350.0f, 0.0f) );

}



// This function returns the x position of the curve dividing
// the two columns of text based on a y position provided as
// input.  x = f(y)
// The function f is dependent on the current value of column split
// as well as the current column style.
float dividingLine(float y) {
	
	float baseWidth = columnSplit + windowSizeX/2;

	switch (columnStyle) {

		case STRAIGHT:
			return baseWidth;
			
		case SINUSOID:
			return baseWidth + 25.0f * sin( 2.0f * (2.0f * M_PI / windowSizeY) * y );

		case TRIANGLE: {
			float halfWinY = (float)windowSizeY / 2.0f;
			if (y < halfWinY)
				return baseWidth + 50.0f * y / halfWinY;
			else
				return baseWidth + 50.0f * (1.0f - ((y - halfWinY) / halfWinY));
		}

		case DIAGONAL:
			return baseWidth - 40 + 80 * (y / windowSizeY);
	}

	return 0.0f;
}



// This method draws te outline of the regions of the screen that should
// hold text.  It serves as both an example of how to compute the column
// width at a certain y position on screen, but can also be a useful 
// visualization when debugging your world placement and line breaking
// algorithm.
void drawGuidelines() {

	int samples = 500.0f;

	float leftMargin = marginSize;
	float rightMargin = (float)windowSizeX - marginSize;
	float bottomMargin = marginSize;
	float topMargin = (float)windowSizeY - marginSize;

	glColor3f(1.0f, 0.0f, 0.0f);
	glLineWidth(2.0f);	

	// draw the border of the left column
	glBegin(GL_LINE_LOOP);

	glVertex2f(leftMargin, topMargin);
	glVertex2f(leftMargin, bottomMargin);
	
	for (int i=0; i<samples; i++) {
		float y = bottomMargin + ((float)i / (samples-1)) * (topMargin-bottomMargin);

		glVertex2f( dividingLine( y ) - marginSize, y);
	}

	glEnd();

	// draw the border of the right column
	glBegin(GL_LINE_LOOP);

	glVertex2f(rightMargin, bottomMargin);
	glVertex2f(rightMargin, topMargin);
	
	for (int i=0; i<samples; i++) {
		float y = bottomMargin + (1.0f-((float)i / (samples-1))) * (topMargin-bottomMargin);

		glVertex2f( dividingLine( y ) + marginSize, y);
	}

	glEnd();
}


static void display() {

	glClearColor(.1f, .1f, .1f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	scene.Draw();

	if (displayGuidelines)
		drawGuidelines();

	glutSwapBuffers();

}


// Keyboard event handler that handles "special keys".  In this app,
// This callback is added to allow the dividing line to be moved with
// the arrow keys.
static void special(int key, int x, int y) {

	switch (key) {

		case GLUT_KEY_LEFT:
			columnSplit -= 2.0f;
			if (columnSplit < -windowSizeX/2.0f + marginSize)
				columnSplit = -windowSizeX/2.0f + marginSize;

			glutPostRedisplay();
			break;

		case GLUT_KEY_RIGHT:
			columnSplit += 2.0f;
			if (columnSplit > windowSizeX/2.0f - marginSize )
				columnSplit = windowSizeX/2.0f - marginSize;

			glutPostRedisplay();
			break;
	}
}


// Keyboard event handler:
// Note, the arrow keys also have functionality in this app, see
// the above function special()
static void keyboard(unsigned char key, int x, int y) {

	switch (key) {

		case '-':
		case '_':

			//
			// CS148 TODO
			// decrease paragraph font size by 1 point here
			// 
			break;

		case '+':
		case '=':

			//
			// CS148 TODO
			// increase paragraph font size by 1 point here
			// 
			break;

		// toggle the column style
		case 'c':
		case 'C':
			columnStyle = static_cast<ColumnStyle>( static_cast<int>(columnStyle) + 1);
			if (columnStyle == INVALID)
				columnStyle = STRAIGHT;

			printf("Changed column style to %d\n", columnStyle);
			glutPostRedisplay();
			break;

		// toggle display of guidelines
		case 'g':
		case 'G':
			displayGuidelines = !displayGuidelines;
			glutPostRedisplay();
			break;

		// quit
		case char(27):
		case 'q':
		case 'Q':
			exit(0);
	}
}


// Respond to reshape event by changing the clipping bounds.
// In this application, the reshape handler ensures that there is
// a unit change in coordinate space corresponds exactly to a single
// pixel on screen.  Since the STText class reports rendered text
// box sizes in units of pixels, making pixels be the same size as 
// a unit step in coordinate space makes things much simpler.
void reshape(int w, int h) {

	windowSizeX = w;
	windowSizeY = h;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	glOrtho(0.0f, (float)w, 0.0f, (float)h, -1.0f, 1.0f);
	glViewport(0, 0, w, h);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	reflowText();

	glutPostRedisplay();
}


int main(int argc, char** argv) {
	

	// Check if source text filename was specified via commandline
	if (argc > 1) {
		strcpy(wordsFilename, argv[1]);
	} else {
		strcpy(wordsFilename, "holmes.txt");
	}

	// Check if font filename was specified via commandline
	if (argc > 2) {
		strcpy(fontFilename, argv[2]);
	} else {
		strcpy(fontFilename, "georgia.ttf");
	}

	printf("Using words from '%s'\n", wordsFilename);
	printf("using font file '%s'\n", fontFilename);

	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowPosition(20,20);
	glutInitWindowSize(512, 512);
	glutCreateWindow("CS148 - Assignment 2");

	glEnable(GL_DEPTH_TEST);

	parseText();

	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(special);
	glutReshapeFunc(reshape);
	glutMainLoop();

	return 0;
}

