
#include "STTransform.h"


/**
* Convenience method. Creates and returns a transform corresponding to the
* specified axis-angle rotation
*/
STTransform
STTransform::Rotate(float theta, float x, float y, float z) {
	STTransform xform;
	xform.SetRotate(theta, x, y, z);
	return xform;
}


/**
* Convenience method. Creates and returns a transform corresponding to the
* specified translation
*/
STTransform
STTransform::Translate(float xtrans, float ytrans, float ztrans) {
	STTransform xform;
	xform.SetTranslate(xtrans, ytrans, ztrans);
	return xform;
}

/**
* Initialize transform to the identity
*/
STTransform::STTransform() {
	mRotTheta = 0.0f;
	mRotAxis[0] = mRotAxis[1] = mRotAxis[2] = 0.0f;
	mTranslate[0] = mTranslate[1] = mTranslate[2] = 0.0f;
	mScale[0] = mScale[1] = mScale[2] = 1.0f;
}


STTransform::STTransform(const STTransform& xform) {
	mRotTheta = xform.mRotTheta;
	for (int i=0; i<3; i++) {
		mRotAxis[i] = xform.mRotAxis[i];
		mTranslate[i] = xform.mTranslate[i];
		mScale[i] = xform.mScale[i];
	}
}

/**
* Specifies an axis-angle rotation component to this transform
*/
void
STTransform::SetRotate(float theta, float x, float y, float z) {
	mRotTheta = theta;
	mRotAxis[0] = x;
	mRotAxis[1] = y;
	mRotAxis[2] = z;
}

/**
* Returns the axis-angle rotation component to this transform
*/
void
STTransform::GetRotate(float& theta, float& x, float& y, float& z) {
	theta = mRotTheta;
	x = mRotAxis[0];
	y = mRotAxis[1];
	z = mRotAxis[2];
}

/**
* Specifies the translation component to this transform
*/
void
STTransform::SetTranslate(float xtrans, float ytrans, float ztrans) {
	mTranslate[0] = xtrans;
	mTranslate[1] = ytrans;
	mTranslate[2] = ztrans;
}

/**
* Returns the translation component to this transform
*/
void
STTransform::GetTranslate(float& xtrans, float& ytrans, float& ztrans) {
	xtrans = mTranslate[0];
	ytrans = mTranslate[1];
	ztrans = mTranslate[2];
}


/**
* Specifies a scale component to this transform
*/
void
STTransform::SetScale(float xscale, float yscale, float zscale) {
	mScale[0] = xscale;
	mScale[1] = yscale;
	mScale[2] = zscale;
}


/**
* Specifies a scale component to this transform (uniform scale in all directions)
*/
void
STTransform::SetScale(float scale) {
	mScale[0] = scale;
	mScale[1] = scale;
	mScale[2] = scale;	
}


/**
* Returns the scale component to this transform
*/
void
STTransform::GetScale(float& xscale, float& yscale, float& zscale) {
	xscale = mScale[0];
	yscale = mScale[1];
	zscale = mScale[2];
}

