
#ifndef __ST_H__
#define __ST_H__


// not defined in math.h on windows
#ifndef M_PI
#define M_PI   3.14159265
#endif


enum STStatus {ST_OK=0, ST_ERROR}; 


/**
 *	All Objects that are to be inserted into a scene must implement
 *	the STDrawable interface
*/
class STDrawable {

	public:
		virtual void Draw() = 0;
};


/**
*  Simple class representing a color with 3 floating point values
*  Value of each channel must remain between 0.0f and 1.0f
*/
class STColor {
	public:
	float r, g, b;

	STColor(float pr, float pg, float pb)
		: r(pr), g(pg), b(pb) { }

	STColor()
		: r(1.0f), g(1.0f), b(1.0f) { }

	STColor(const STColor& c)
		: r(c.r), g(c.g), b(c.b)
	{ }

	const STColor operator*(float scalar) {
		return STColor( r * scalar, g * scalar, b * scalar );
	}
};


#include "STImage.h"
#include "STShape.h"
#include "STTransform.h"
#include "STSceneNode.h"
#include "STTimer.h"
#include "STText.h"
#include "STFontFace.h"


#endif  // ST_H___
